searchNodes=[{"doc":"A module that contains a function to test the dispatch of forms. This will make it easier to write integration tests to check if forms in Phoenix templates will submit to the intended controller action with the right params.","ref":"TestDispatch.html","title":"TestDispatch","type":"module"},{"doc":"Will find a form in the HTML response of the given conn by entity or by TestSelector, or, if no entity or test_selector is provided, it will target the last form found in the response. Next it will look for form controls (inputs, selects), convert these to params and use the attributes passed to dispatch_form_with/3 to update the values of the params. The params will now only contain field keys found in the controls of the form. If an entity is given, the params will be prepended by this entity. Ultimately, the conn is dispatched to the conn&#39;s private.phoenix_endpoint using Phoenix.ConnTest.dispatch/5, with the params and with the method and action found in the form.","ref":"TestDispatch.html#dispatch_form_with/3","title":"TestDispatch.dispatch_form_with/3","type":"function"},{"doc":"Finds a link by a given conn, test_selector and an optional test_value. Hereby it tries to get a response from the conn and find the first &lt;a&gt;&lt;/a&gt; element that has the combination of the test_selector and test_value. The link that is found will be dispatched with Phoenix.ConnTest.dispatch/4. The method will be derived from the link by the data-method attribute and has &quot;get&quot; as default. The path will be taken from the href. Examples With the given page on &quot;/posts/1&quot; &lt;html&gt; &lt;body&gt; &lt;h1&gt;Post&lt;/h1&gt; &lt;a href=&quot;/posts/1&quot; data-method=&quot;delete&quot; test-selector=&quot;post-123-delete-post&quot;&gt; Remove &lt;/a&gt; &lt;table&gt; &lt;th&gt;Comment&lt;/td&gt; &lt;th&gt;Upvote&lt;/td&gt; &lt;tr&gt; &lt;td&gt;This is perfect&lt;/td&gt; &lt;td&gt; &lt;a href=&quot;/posts/1/comments/1&quot; data-method=&quot;post&quot; test-value=&quot;1&quot; test-selector=&quot;post-123-upvote-comment&quot; &gt; Upvote Comment &lt;/td&gt; &lt;/tr&gt; &lt;/table&gt; &lt;/body&gt; &lt;/html&gt; iex&gt; conn = build_conn() |&gt; get(&quot;/posts/1&quot;) iex&gt; result = dispatch_link(conn, &quot;post-123-delete-post&quot;) iex&gt; with %Plug.Conn{request_path: &quot;/posts/1&quot;, method: &quot;DELETE&quot;} &lt;- result, do: :ok :ok iex&gt; conn = build_conn() |&gt; get(&quot;/posts/1&quot;) iex&gt; result = dispatch_link(conn, &quot;post-123-upvote-comment&quot;, &quot;1&quot;) iex&gt; with %Plug.Conn{request_path: &quot;/posts/1/comments/1&quot;, method: &quot;POST&quot;} &lt;- result, do: :ok :ok","ref":"TestDispatch.html#dispatch_link/3","title":"TestDispatch.dispatch_link/3","type":"function"},{"doc":"Testing with TestDispatch This library was built to test if forms and links have the expected results in controller tests when they would be dispatched. It can be used with or without the implementation of TestSelector.","ref":"testing-with-test-dispatch.html","title":"Testing with TestDispatch","type":"extras"},{"doc":"We will explain the workings with the following form. &lt;form action=&quot;admin/users/create&quot; method=&quot;post&quot;&gt; &lt;input name=&quot;_csrf_token&quot; type=&quot;hidden&quot; value=&quot;AHIqJmEUAxIvGy4HJ3oUGCMjChsLYBZ-SGgy7W1HElh3PKLsffgXXQO6&quot;&gt; &lt;label&gt;Name&lt;/label&gt; &lt;input id=&quot;user_name&quot; label=&quot;Name&quot; name=&quot;user[name]&quot; type=&quot;text&quot;&gt; &lt;label&gt;Email&lt;/label&gt; &lt;input id=&quot;user_email&quot; label=&quot;Email&quot; name=&quot;user[email]&quot; type=&quot;email&quot;&gt; &lt;label&gt;Description&lt;/label&gt; &lt;textarea id=&quot;user_description&quot; label=&quot;Description&quot; name=&quot;user[description]&quot; type=&quot;textarea&quot;&gt;&lt;/textarea&gt; &lt;select id=&quot;user_role&quot; name=&quot;user[role]&quot;&gt; &lt;option value=&quot;&quot;&gt;Select a role&lt;/option&gt; &lt;option value=&quot;admin&quot;&gt;Admin&lt;/option&gt; &lt;option value=&quot;moderator&quot;&gt;Moderator&lt;/option&gt; &lt;/select&gt; &lt;button type=&quot;submit&quot;&gt;Create new user&lt;/button&gt; &lt;/form&gt; This form is shown on /admin/users/new the entity for this form is user. To post the form as is we can use the following code: conn |&gt; get(&quot;/admin/users/new&quot;) |&gt; disptach_form_with(:user) |&gt; html_response(422) =~ &quot;can&#39;t be blank&quot; A request is made to the router here with the method POST to the action /admin/users/create and with the params %{&quot;user&quot; =&gt; %{&quot;name&quot; =&gt; &quot;&quot;, &quot;email&quot; =&gt; &quot;&quot;, &quot;description&quot; =&gt; &quot;&quot;, &quot;roles&quot; =&gt; &quot;&quot;}} The dispatch_form_with/3 will return a conn with the response of the controller. In this case it has returned an error because all fields are left blank. To update the fields while posting, a map can be given that matches the fields. params = %{ name: &quot;Marcel&quot;, email: &quot;marcel@example.com&quot;, description: &quot;Securing this place&quot;, role: &quot;moderator&quot; } conn |&gt; get(&quot;/admin/users/new&quot;) |&gt; disptach_form_with(params, :user) |&gt; redirected_to(302) == &quot;/admin/users/2&quot; Now that the params are given each key is matched to the keys in the form and updated with the value that is provided. If the keys do not match they won&#39;t be posted. TestSelector Some forms can be created that only have a submit button and cannot be found with an entity. In this we can use TestSelector to find the form and dispatch it. &lt;form action=&quot;admin/posts/1&quot; method=&quot;delete&quot; test-selector=&quot;posts-123-delete-post&quot;&gt; &lt;input name=&quot;_csrf_token&quot; type=&quot;hidden&quot; value=&quot;AHIqJmEUAxIvGy4HJ3oUGCMjChsLYBZ-SGgy7W1HElh3PKLsffgXXQO6&quot;&gt; &lt;button type=&quot;submit&quot;&gt;Remove&lt;/button&gt; &lt;/form&gt; This form can be dispatched with conn |&gt; get(&quot;/admin/posts&quot;) |&gt; dispatch_form_with(&quot;post-123-delete-post&quot;) |&gt; html_response(200) =~ &quot;Post is deleted&quot; More documentation on how to use TestSelector can be found in TestSelectors wiki","ref":"testing-with-test-dispatch.html#testing-a-form","title":"Testing with TestDispatch - Testing a form","type":"extras"},{"doc":"Links can also be dispatched and currently we can only do this by using TestSelector. For this dispatch_link/3 can be used. For this example we can take a posts show page on /posts/1 &lt;div&gt; &lt;h1&gt;Post&lt;/h1&gt; &lt;a href=&quot;/posts/1&quot; data-method=&quot;delete&quot; test-selector=&quot;post-123-delete-post&quot;&gt; Remove &lt;/a&gt; &lt;table&gt; &lt;tr&gt; &lt;td&gt;This is perfect&lt;/td&gt; &lt;td&gt; &lt;a href=&quot;/posts/1/comments/1&quot; data-method=&quot;post&quot; test-value=&quot;1&quot; test-selector=&quot;post-123-upvote-comment&quot; &gt; Upvote Comment &lt;/td&gt; &lt;/tr&gt; &lt;tr&gt; &lt;td&gt;A better comment&lt;/td&gt; &lt;td&gt; &lt;a href=&quot;/posts/1/comments/2&quot; data-method=&quot;post&quot; test-value=&quot;1&quot; test-selector=&quot;post-123-upvote-comment&quot; &gt; Upvote Comment &lt;/td&gt; &lt;/tr&gt; &lt;/table&gt; &lt;/div&gt; We can now delete this post in the controller test by doing: conn |&gt; get(&#39;/posts/1&#39;) |&gt; dispatch_link(&quot;post-123-delete-post&quot;) |&gt; html_response(200) =~ &quot;Post was deleted&quot; The dispatch_link parses the page and tries to find post-123-delete-post it take the method by the data-method attribute. The url to dispatch to is taken from the href. In this case a delete request is done to posts/1. To take a specific element from a list test-values can be used as third argument of dispatch_link/3. conn |&gt; get(&#39;/posts/1&#39;) |&gt; dispatch_link(&quot;post-123-upvote-comment&quot;, &quot;1&quot;) |&gt; html_response(200) =~ &quot;Upvoted comment 2&quot; As expected here a post request is done to /posts/1/comments/2 for the second comment. If there is no data-method set dispatch_link/3 will do a get request by default.","ref":"testing-with-test-dispatch.html#testing-links","title":"Testing with TestDispatch - Testing Links","type":"extras"}]